/**
 * GMyth Library
 *
 * @file gmyth/gmyth_settings.c
 * 
 * @brief <p> This component contains functions acessing and modifying 
 * user and program settings.
 * 
 * The standard settings file is created in the user home folder, in the
 * file ~/.gmyth/settings.dat.  
 *
 * Copyright (C) 2006 INdT - Instituto Nokia de Tecnologia.
 * @author Alexsandro Jose Virginio dos Santos <alexsandro.santos@indt.org.br>
 *
 *//*
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "gmyth_settings.h"
 
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

#include <glib.h>
#include <glib/gprintf.h>
#include <glib/gstdio.h>

#define GMYTH_SETTINGS_FILE_NAME	"settings.dat"
#define GMYTH_SETTINGS_DIR		".gmyth"

static void gmyth_settings_class_init          (GMythSettingsClass *klass);
static void gmyth_settings_init                (GMythSettings *object);

static void gmyth_settings_dispose  (GObject *object);
static void gmyth_settings_finalize (GObject *object);

static void fill_settings_data(GMythSettings *gmyth_settings, char* line);

G_DEFINE_TYPE(GMythSettings, gmyth_settings, G_TYPE_OBJECT)

static void
gmyth_settings_class_init (GMythSettingsClass *klass)
{
	GObjectClass *gobject_class;

    gobject_class = (GObjectClass *) klass;

    gobject_class->dispose  = gmyth_settings_dispose;
    gobject_class->finalize = gmyth_settings_finalize;	
}

static void
gmyth_settings_init (GMythSettings *gmyth_settings)
{

	gmyth_settings->settings_file = NULL;
	
    gmyth_settings->backend_hostname = g_string_new("127.0.0.1");
    gmyth_settings->backend_port = 6543;
    gmyth_settings->mysql_dbname = g_string_new("mythconverg");
	gmyth_settings->mysql_username = g_string_new("mythtv");
    gmyth_settings->mysql_password = g_string_new("mythtv");
    gmyth_settings->database_type = g_string_new("mysql");
    	
}


static void
gmyth_settings_dispose  (GObject *object)
{
	GMythSettings *gmyth_settings = GMYTH_SETTINGS(object);

	if ( gmyth_settings->backend_hostname != NULL )
		g_string_free( gmyth_settings->backend_hostname, TRUE );
	if ( gmyth_settings->mysql_dbname != NULL )
		g_string_free( gmyth_settings->mysql_dbname, TRUE );
	if ( gmyth_settings->mysql_username != NULL )
		g_string_free( gmyth_settings->mysql_username, TRUE );
	if ( gmyth_settings->mysql_password != NULL )
		g_string_free( gmyth_settings->mysql_password, TRUE );
	if ( gmyth_settings->database_type != NULL )
		g_string_free( gmyth_settings->database_type, TRUE );

	G_OBJECT_CLASS (gmyth_settings_parent_class)->dispose (object);
}

static void
gmyth_settings_finalize (GObject *object)
{
	g_signal_handlers_destroy (object);

    G_OBJECT_CLASS (gmyth_settings_parent_class)->finalize (object);
}

/** Creates a new instance of GMythSettings.
 * 
 * @return a new instance of GMythSettings.
 */
GMythSettings*
gmyth_settings_new (void)
{
    GMythSettings *gmyth_settings = GMYTH_SETTINGS (g_object_new(GMYTH_SETTINGS_TYPE, NULL));

    return gmyth_settings;
}

static gboolean
gmyth_settings_ensure_dir_exists (const gchar *dir)
{

	g_debug ("[%s] \tdir = %s\n", __FUNCTION__, dir);

    if (g_file_test (dir, G_FILE_TEST_IS_DIR) == FALSE) {
	    if (g_file_test (dir, G_FILE_TEST_EXISTS) == TRUE) {
            g_warning ("%s exists, please move it out of the way.", dir);
            return FALSE;
	    }

        if (mkdir (dir, 488) != 0) {
            g_warning ("Failed to create directory %s.", dir);
            return FALSE;
        }
    }

    return TRUE;
}

static gboolean
gmyth_settings_ensure_file_exists (const gchar *file_name) {

	int file = 0;
	
    if ( g_file_test( file_name, G_FILE_TEST_EXISTS ) == FALSE ) {
        g_debug ( "\n\tCreating %s file...\n", file_name );
        file = creat( file_name, S_IRWXU | S_IRWXO | S_IRWXG );

		if ( close( file ) == -1 )
			return FALSE;
    }

	return TRUE;
}

/** Loads the GMyth settings from the given file.
 * 
 * @param gmyth_settings the GMythSettings instance.
 * @param filename The desired file to be opened.
 * @return TRUE if success, FALSE if error.
 */
gboolean
gmyth_settings_load_from_file (GMythSettings *gmyth_settings, GString *filename)
{
    FILE *file_desc;
    char line[100];
	
	g_debug ("[%s] Loading GMyth settings file: %s", __FUNCTION__, filename->str);
	
    file_desc = fopen(filename->str, "r");
    if( file_desc == NULL) {
        g_warning ("[%s] Settings file %s could not be opened", __FUNCTION__, filename->str);
        return FALSE;
    }
	
	gmyth_settings->settings_file = g_string_new (filename->str);
	
    while(fgets(line, 100, file_desc)) {
        int i;

        /* Removes the new line characters from the end of line */
        for ( i = strlen(line)-1; i >= 0; i--) {
            if ( !g_ascii_iscntrl (line[i]) ) {
                line[i+1] = '\0';
                break;
            }
        }
        
        fill_settings_data(gmyth_settings, line);
    }

    fclose (file_desc);

    return TRUE;
}

/** Loads the GMyth settings from the standard settings file
 * (~/.gmyth/settings.dat).
 * 
 * @param gmyth_settings the GMythSettings instance.
 * @return TRUE if success, FALSE if error.
 */
gboolean
gmyth_settings_load (GMythSettings *gmyth_settings)
{
	GString* file_full_path = g_string_new( g_build_filename( g_get_home_dir(), 
				GMYTH_SETTINGS_DIR, GMYTH_SETTINGS_FILE_NAME, NULL ) );

	gmyth_settings->settings_file = file_full_path;
		
	gmyth_settings_ensure_dir_exists( g_build_filename( g_get_home_dir(), GMYTH_SETTINGS_DIR, NULL ) );

	// Verifies if the file already exist
	if ( g_file_test( file_full_path->str, G_FILE_TEST_EXISTS ) == FALSE ) {
		g_debug ("[%s] Settings file does not exist. A new one will be created.\n", __FUNCTION__);

		if ( gmyth_settings_ensure_file_exists( file_full_path->str ) == FALSE )
			return FALSE;

		// Creates the file with default values (see _init function)
		return gmyth_settings_save (gmyth_settings);
		
	} else {
		g_debug ("[%s] Opening settings from file %s", __FUNCTION__, file_full_path->str);
		
		return gmyth_settings_load_from_file (gmyth_settings, file_full_path);
	}	
}	

/** Saves the current gmyth_settings to the settings file related to the
 * given instance.
 * 
 * @param gmyth_settings the GMythSettings instance.
 * @return TRUE if success, FALSE if error.
 */
gboolean
gmyth_settings_save (GMythSettings *gmyth_settings)
{
    FILE *file_desc;
	
	if (gmyth_settings->settings_file == NULL) {
		g_warning ("[%s] Settings were not loaded from file, could not save!", __FUNCTION__);
	}
	
    file_desc = fopen(gmyth_settings->settings_file->str, "w");
    if( file_desc == NULL) {
        g_warning ("GMYTH_SETTINGS: settings file %s not found", gmyth_settings->settings_file->str);
        return FALSE;
    }
    
	g_fprintf(file_desc, "#Maemo-Myth Settings\n");

	g_fprintf(file_desc, "#General settings related with mythtv database\n");
	g_fprintf(file_desc, "dbname=%s\n", gmyth_settings->mysql_dbname->str);
	g_fprintf(file_desc, "username=%s\n", gmyth_settings->mysql_username->str);
	g_fprintf(file_desc, "password=%s\n", gmyth_settings->mysql_password->str);

	g_fprintf(file_desc, "\n#General settings related with mythtv backend\n");
	g_fprintf(file_desc, "hostname=%s\n", gmyth_settings->backend_hostname->str);
	g_fprintf(file_desc, "backend_port=%d\n", gmyth_settings->backend_port);

    fclose (file_desc);

	g_debug ("[%s] Settings file saved", __FUNCTION__);
    return TRUE;
}

/** Gets the backend hostname from the settings.
 * 
 * @param gmyth_settings the GMythSettings instance.
 * @return The loaded backend hostname, or the default value "127.0.0.1" if settings
 * file was not opened.
 */
GString*
gmyth_settings_get_backend_hostname (GMythSettings *gmyth_settings)
{
	return g_string_new (gmyth_settings->backend_hostname->str);
}

/** Sets the backend hostname to the settings.
 * 
 * @param gmyth_settings the GMythSettings instance.
 * @param new_hostname The new hostname.
 */
void
gmyth_settings_set_backend_hostname (GMythSettings *gmyth_settings, GString *new_hostname)
{
	g_string_assign (gmyth_settings->backend_hostname, new_hostname->str);
}

/** Gets the user name to connect to backend database.
 * 
 * @param gmyth_settings the GMythSettings instance.
 * @return The loaded user name, or the default value "mythtv" if settings
 * file was not opened.
 */
GString*
gmyth_settings_get_username (GMythSettings *gmyth_settings)
{
	return g_string_new (gmyth_settings->mysql_username->str);
}

/** Sets the username to connect to backend database.
 * 
 * @param gmyth_settings the GMythSettings instance.
 * @param new_username The new username.
 */
void
gmyth_settings_set_username (GMythSettings *gmyth_settings, GString *new_username)
{
	g_string_assign (gmyth_settings->mysql_username, new_username->str);
}

/** Gets the database password from the settings file.
 * 
 * @param gmyth_settings the GMythSettings instance.
 * @return The loaded password, or the default value "mythtv" if settings
 * file was not opened.
 */
GString*
gmyth_settings_get_password (GMythSettings *gmyth_settings)
{
	return g_string_new (gmyth_settings->mysql_password->str);	
}

/** Sets the database password.
 * 
 * @param gmyth_settings the GMythSettings instance.
 * @param new_password The new password.
 */

void
gmyth_settings_set_password (GMythSettings *gmyth_settings, GString *new_password)
{
	g_string_assign (gmyth_settings->mysql_password, new_password->str);
}

/** Gets the backend database name from the settings file.
 * 
 * @param gmyth_settings the GMythSettings instance.
 * @return The loaded database name, or the default value "mythconverg" if settings
 * file was not opened.
 */
GString*
gmyth_settings_get_dbname (GMythSettings *gmyth_settings)
{
	return g_string_new (gmyth_settings->mysql_dbname->str);
}

/** Sets the Mythtv database name to the settings.
 * 
 * @param gmyth_settings the GMythSettings instance.
 * @param new_dbname The new database name.
 */

void
gmyth_settings_set_dbname (GMythSettings *gmyth_settings, GString *new_dbname)
{
	g_string_assign (gmyth_settings->mysql_dbname, new_dbname->str);
}

/** Gets the backend port from the settings.
 * 
 * @param gmyth_settings the GMythSettings instance.
 * @return The loaded backend port, or the default value "6543" if settings
 * file was not opened.
 */
int
gmyth_settings_get_backend_port (GMythSettings *gmyth_settings)
{
	return gmyth_settings->backend_port;
}

/** Sets the backend port.
 * 
 * @param gmyth_settings the GMythSettings instance.
 * @param new_port The new port.
 */
void
gmyth_settings_set_backend_port (GMythSettings *gmyth_settings, gint new_port)
{
	gmyth_settings->backend_port = new_port;
}


static void
fill_settings_data(GMythSettings *gmyth_settings, char* line)
{
    gchar** str_splited;

	GString *gstr_splited;
	gstr_splited = g_string_new("");
	str_splited = g_strsplit (line, "=", -1);

	if(!(strcmp(str_splited[0], "hostname"))){
		g_string_assign(gstr_splited, str_splited[1]);
		gmyth_settings_set_backend_hostname(gmyth_settings, gstr_splited);
	}
	else if (!(strcmp(str_splited[0], "dbname"))){
		g_string_assign(gstr_splited, str_splited[1]);
		gmyth_settings_set_dbname(gmyth_settings, gstr_splited);	
	}
	else if (!(strcmp(str_splited[0], "username"))){
		g_string_assign(gstr_splited, str_splited[1]);
		gmyth_settings_set_username(gmyth_settings, gstr_splited);
	}	
	else if (!(strcmp(str_splited[0], "password"))){
		g_string_assign(gstr_splited, str_splited[1]);
		gmyth_settings_set_password(gmyth_settings, gstr_splited);
	}
	else if (!(strcmp(str_splited[0], "backend_port"))){
		gmyth_settings_set_backend_port(gmyth_settings, atoi(str_splited[1]));
	}		
}
